#include "variables.h"
#include <p24F16KA101.h>
#include "i2c.h"
#include "timer.h"

void i2c_init() {

	I2C1CON = 0x0000;			// clear reg
	I2C1STAT = 0x0000;			// clear error flags				
	IFS1bits.SI2C1IF = 0;		// clear the MI2C & SI2C Interrupts
	IFS1bits.MI2C1IF = 0;
	IEC1bits.SI2C1IE = 1;   	// enable slave interrupt
	IEC1bits.MI2C1IE = 0;
	I2C1ADD = I2C_ADDR;
	I2C1BRG = 9;				// Fcy = 4 MHz, I2C clock @ 400 KHz
	I2C1CON = 0xB040;			// enable I2C; discontinue in IDLE; clock stretching enabled; 7-bit slave address; 

}

void __attribute__((interrupt, auto_psv)) _SI2C1Interrupt(void) {

	_SI2C1IF = 0;

	I2C1STATbits.I2COV = 0;              	// clear overflow flag at every new byte
	I2C1STATbits.IWCOL = 0;              	// clear collision flag at every new byte

	switch (i2cState) {

		case STATE_WAIT_FOR_ADDR:

	    	I2C1STATbits.I2COV = 0;              	// clear OV flag at every new transaction

			TMR2 = 0;
			T2CONbits.TON = 1;						// start the timer2 for communication timeout

        	i2cRecv = I2C1RCV; 						//clear RBF bit for address
        	if (I2C1STATbits.R_W) {					// read (output from slave)
				dataIndexOut = 9;
				I2C1TRN = dataValues[dataIndexOut]; // get first data byte
                i2cState = STATE_READ_FROM_SLAVE; 	// read transaction
        	} else {								// write (input to slave)
				dataIndexIn = 0;
				i2cState = STATE_WRITE_TO_SLAVE;
			}
			I2C1CONbits.SCLREL = 1;     		// release clock line so MASTER can drive it
			break;     

		case STATE_WRITE_TO_SLAVE:
       		dataValues[dataIndexIn++] = I2C1RCV;	// character arrived, place in buffer; RBF cleared
       		if (dataIndexIn == 9) {					// all new duty cycle values received
         		i2cState = STATE_WAIT_FOR_ADDR; 	// wait for next transaction
				T2CONbits.TON = 0;
       		}
			I2C1CONbits.SCLREL = 1;     		// release clock line so MASTER can drive it
       		break;
     
		case STATE_READ_FROM_SLAVE:
       		I2C1TRN = dataValues[dataIndexOut++];	// just keep placing data in buffer as MASTER reads our I2C port
       		I2C1CONbits.SCLREL = 1;     			// release clock line so MASTER can drive it
       		if (dataIndexOut == 13) {
				i2cState = STATE_READ_FROM_SLAVE_LAST;	// this is the last character, after byte is shifted out, release the clock line again
			}
       		break;
     
		case STATE_READ_FROM_SLAVE_LAST:  //this is interrupt for last character finished shifting out
        	i2cState = STATE_WAIT_FOR_ADDR;
       		I2C1CONbits.SCLREL = 1;     			// release clock line so MASTER can drive it
			T2CONbits.TON = 0;
       		break;
     
		default:
       		i2cState = STATE_WAIT_FOR_ADDR;
   	}


}


