#include "variables.h"
#include <p24F16KA101.h>
#include "timers.h"
#include "i2c.h"

void init_timers() {

	// used to generate a pwm
	T1CON = 0;	
	T1CONbits.TCS = 0;									// internal clock => Fosc/2 = 4 MHz
	T1CONbits.TSIDL = 1;								// Discontinue module operation during IDLE mode
	T1CONbits.TCKPS = 0b00;								// prescaler 1:1 => 4 MHz (0.25 us)
	TMR1 = 0;
	PR1 = 4000000/(LED_PERIOD_FREQ*TICKS_PER_PERIOD);	// (((1/LED_PERIOD_FREQ)/TICKS_PER_PERIOD)*1000000)/0.25 => 4000000/(LED_PERIOD_FREQ*TICKS_PER_PERIOD)
	//IPC0bits.T1IP = 7;									// highest priority
	IFS0bits.T1IF = 0;									// clear interrupt flag
	IEC0bits.T1IE = 1;									// set interrupt enable bit
	T1CONbits.TON = 1;									// start the timer1
	
	// used for timeout
	T2CON = 0;	
	T2CONbits.TCS = 0;									// internal clock => Fosc/2 = 4 MHz
	T2CONbits.TSIDL = 1;								// Discontinue module operation during IDLE mode
	T2CONbits.TCKPS = 0b00;								// prescaler 1:1 => 4 MHz (0.25 us)
	TMR2 = 0;
	PR2 = 40000;										// 10000/0.25=40000 (interrupt/timeout after 5 ms)
	IFS0bits.T2IF = 0;									// clear interrupt flag
	IEC0bits.T2IE = 1;									// set interrupt enable bit	
	T2CONbits.TON = 0;
}

void __attribute__ ((interrupt,auto_psv)) _T1Interrupt(void) {
	
	IFS0bits.T1IF = 0;
	
	if(currentTick < (TICKS_PER_PERIOD-1)) {
		currentTick++;
	} else {
		currentTick=0;
		if(dataValues[DUTYR1] > 0) {
			LEDR1_ON;
		}
		if(dataValues[DUTYG1] > 0) {
			LEDG1_ON;
		}
		if(dataValues[DUTYB1] > 0) {
			LEDB1_ON;
		}
		if(dataValues[DUTYR2] > 0) {
			LEDR2_ON;
		}
		if(dataValues[DUTYG2] > 0) {
			LEDG2_ON;
		}
		if(dataValues[DUTYB2] > 0) {
			LEDB2_ON;
		}
		if(dataValues[DUTYR3] > 0) {
			LEDR3_ON;
		}
		if(dataValues[DUTYG3] > 0) {
			LEDG3_ON;
		}
		if(dataValues[DUTYB3] > 0) {
			LEDB3_ON;
		}
	}	
	
	if(dataValues[DUTYR1] == currentTick) {
		LEDR1_OFF;
	}
	if(dataValues[DUTYG1] == currentTick) {
		LEDG1_OFF;
	}
	if(dataValues[DUTYB1] == currentTick) {
		LEDB1_OFF;
	}

	if(dataValues[DUTYR2] == currentTick) {
		LEDR2_OFF;
	}
	if(dataValues[DUTYG2] == currentTick) {
		LEDG2_OFF;
	}
	if(dataValues[DUTYB2] == currentTick) {
		LEDB2_OFF;
	}	

	if(dataValues[DUTYR3] == currentTick) {
		LEDR3_OFF;
	}
	if(dataValues[DUTYG3] == currentTick) {
		LEDG3_OFF;
	}
	if(dataValues[DUTYB3] == currentTick) {
		LEDB3_OFF;
	}

}

void __attribute__ ((interrupt,auto_psv)) _T2Interrupt(void) {
	
	IFS0bits.T2IF = 0;
	i2c_init();
	i2cState = STATE_WAIT_FOR_ADDR;
	
}

